==> team4958_customs-0.1.0-py3-none-any.whl: team4958_customs/utilities/project_builder.py <==
"""
if you are tired of recreating same project structures again and again
--------
"""



from team4958_customs.utilities.assets.builder_presets import *
from team4958_customs.utils import ROOT, MISSING, _yesornot

import pathlib
from pathlib import Path

import os


__all__ = [
    'Build'
]



NUMBERS = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '0']
LOWERCASES = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z']
UPPERCASES = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z']
ALLOWED_SYM = ['_']
UNRECOMMENDED_SYM = [' ','(',')','-',"'",';']

RESTRICTED = ['`', '~', '!', '@', '#', '$','^','&','*','=','+','?','{','}',':','<',',','.','>','№','|','/','\\','"']
ALLOWED = NUMBERS+LOWERCASES+ALLOWED_SYM
UNRECOMMENDED = UPPERCASES+UNRECOMMENDED_SYM



class _Check:
    def sym_presence(string:str, sym_list:list):
        """
        checks if symbols from listing are present in string
        """
        res = []
        string = list(set(list(string)))
        for sym in string:
            if sym in sym_list:
                res.append(sym)
        if res!=[]:
            return res
        else:
            return



def _projectname():
    name = input("give your project a proper name: ")
    restricted = _Check.sym_presence(name, RESTRICTED)
    unrecommended = _Check.sym_presence(name, UNRECOMMENDED)
    if name=='':
        print('name cannot be empty!')
        return _projectname()
    elif restricted:
        print(f"there are some restricted symbols in project name given: {restricted}")
        return _projectname()
    elif unrecommended:
        confirm = _yesornot(f'there are some unrecommended symbols in project name!\nsymbols checker didnt like: {unrecommended}\ndo you really wish to keep it like that?')
        if confirm:
            return name
        else:
            return _projectname()
    else:
        return name



class Build:
    """builds a project template from preset inside a subfolder named as your project"""

    def blank(name: str=MISSING, create_subfolder=True, readme=True, requirements=True, license_file=False):
        """
        blank preset which creates `src` folder and `main.py` file without anything inside
        ---------
        \n
        also creates empty `README.md`, `requirements.txt` and `LICENSE` files\n
        ---------\n
        name: name of your project folder (if not specified inside the code will be asked through console)\n
        required when create_subfolder is 'True' (which is deafult)\n
        --------------\n
        create_subfolder: whether you wish to create a subfolder for your project (builds template in a current directory if 'False')\n
        -------\n
        readme: whether you wish to create a `README.md` file in your project's root directory\n
        -------\n
        requirements: whether you wish to create a `requirements.txt` file in your project's root directory\n
        ---------\n
        license_file: whether you wish to create a `LICENSE` file in your project's root directory (will be empty)
        """
        if name is MISSING and create_subfolder==True:
            name = _projectname()
        os.mkdir(Path(ROOT, name, 'src'))
        with open(Path(ROOT, name, 'main.py'), 'w') as f:
            pass
        if readme:
            with open(Path(ROOT, name, 'README.md'), 'w') as f:
                pass
        if requirements:
            with open(Path(ROOT, name, 'requirements.txt'), 'w') as f:
                f.write("team4958_customs")
        if license_file:
            with open(Path(ROOT, name, 'LICENSE'), 'w') as f:
                pass
    
    def _fromdict(obj:dict, path=MISSING):
        """
        `NEVER FUCKING CALL THIS INSIDE YOUR CODE`
        ------------
        \n
        this will spawn your whole structure directly in current work directory!\n
        `use 'fromdict' method instead`\n
        --------\n
        builds a project structure from dict
        ----
        \n
        -------\n
        example_dict = {
            'folder_name':{
                #feel free to make as much substructures as you need
            },
            'filename.extension':['line1', 'line2', 'line3', etc.],
            'filename2.extension':'any text to put inside the file just as string'
        }
        """
        if path is MISSING:
            path = ROOT
        for (key, val) in obj.items():
            if type(val) is dict:
                os.mkdir(pathlib.Path(path, key))
                newpath = pathlib.Path(path, key)
                Build.fromdict(val, newpath)
            elif type(val) is list or type(val) is str:
                with open(pathlib.Path(path, key), 'w') as f:
                    if type(val) is list:
                        if val!=[]:
                            for line in val:
                                f.writelines(line)
                    else:
                        f.write(val)
            else:
                print(f'"{key}" type not specified as file or folder, skipping...')
    
    def fromdict(preset:dict, name: str=MISSING, create_subfolder=True):
        """
        builds a project structure from dict
        ----
        \n
        ---------\n
        preset: your own structure preset presented as dictionary
        \n
        example_dict = {\n
            'folder_name':{\n
                #feel free to make as much substructures as you need\n
            },\n
            'filename.extension':['line1', 'line2', 'line3', etc.],\n
            'filename2.extension':'any text to put inside the file just as string'\n
        }\n
        -------\n
        name: your project's name (required and has its point only if 'create_subfolder' is True)\n
        create_subfolder: whether you wish to create everything inside a subfolder\n
        """
        if name is MISSING and create_subfolder==True:
            name = _projectname()
            os.mkdir(Path(ROOT, name))
            proj_path = Path(ROOT, name)
        else:
            proj_path = ROOT
        Build._fromdict(preset, proj_path)
    
    def deafult(name: str=MISSING, create_subfolder=True):
        """
        builds a project structure from built-in deafult template
        ----
        \n
        ---------\n
        name: your project's name (required and has its point only if 'create_subfolder' is True)\n
        create_subfolder: whether you wish to create everything inside a subfolder\n
        """
        if name is MISSING and create_subfolder==True:
            name = _projectname()
            os.mkdir(Path(ROOT, name))
            proj_path = Path(ROOT, name)
        else:
            proj_path = ROOT
        Build._fromdict(deafult_preset, path=proj_path)
        
    # def disnake_bot(name: str=MISSING, create_subfolder=True):
    #     """
    #     builds a disnake bot project structure from built-in template
    #     ----
    #     \n
    #     ---------\n
    #     name: your project's name (required and has its point only if 'create_subfolder' is True)\n
    #     create_subfolder: whether you wish to create everything inside a subfolder\n
    #     """
    #     if name is MISSING and create_subfolder==True:
    #         name = _projectname()
    #         os.mkdir(Path(ROOT, name))
    #         proj_path = Path(ROOT, name)
    #     else:
    #         proj_path = ROOT
    #     Build._fromdict(disnake_preset, path=proj_path)

==> team4958_customs-0.1.1-py3-none-any.whl: team4958_customs/utilities/project_builder.py <==
"""
if you are tired of recreating same project structures again and again
--------
"""



from team4958_customs.utilities.assets.builder_presets import *
from team4958_customs.utils import ROOT, MISSING, _yesornot

import pathlib
from pathlib import Path

import os


__all__ = [
    'Build'
]



NUMBERS = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '0']
LOWERCASES = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z']
UPPERCASES = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z']
ALLOWED_SYM = ['_']
UNRECOMMENDED_SYM = [' ','(',')','-',"'",';']

RESTRICTED = ['`', '~', '!', '@', '#', '$','^','&','*','=','+','?','{','}',':','<',',','.','>','№','|','/','\\','"']
ALLOWED = NUMBERS+LOWERCASES+ALLOWED_SYM
UNRECOMMENDED = UPPERCASES+UNRECOMMENDED_SYM



class _Check:
    def sym_presence(string:str, sym_list:list):
        """
        checks if symbols from listing are present in string
        """
        res = []
        string = list(set(list(string)))
        for sym in string:
            if sym in sym_list:
                res.append(sym)
        if res!=[]:
            return res
        else:
            return



def _projectname():
    name = input("give your project a proper name: ")
    restricted = _Check.sym_presence(name, RESTRICTED)
    unrecommended = _Check.sym_presence(name, UNRECOMMENDED)
    if name=='':
        print('name cannot be empty!')
        return _projectname()
    elif restricted:
        print(f"there are some restricted symbols in project name given: {restricted}")
        return _projectname()
    elif unrecommended:
        confirm = _yesornot(f'there are some unrecommended symbols in project name!\nsymbols checker didnt like: {unrecommended}\ndo you really wish to keep it like that?')
        if confirm:
            return name
        else:
            return _projectname()
    else:
        return name



class Build:
    """builds a project template from preset inside a subfolder named as your project"""

    def blank(name: str=MISSING, create_subfolder=True, readme=True, requirements=True, license_file=False):
        """
        blank preset which creates `src` folder and `main.py` file without anything inside
        ---------
        \n
        also creates empty `README.md`, `requirements.txt` and `LICENSE` files\n
        ---------\n
        name: name of your project folder (if not specified inside the code will be asked through console)\n
        required when create_subfolder is 'True' (which is deafult)\n
        --------------\n
        create_subfolder: whether you wish to create a subfolder for your project (builds template in a current directory if 'False')\n
        -------\n
        readme: whether you wish to create a `README.md` file in your project's root directory\n
        -------\n
        requirements: whether you wish to create a `requirements.txt` file in your project's root directory\n
        ---------\n
        license_file: whether you wish to create a `LICENSE` file in your project's root directory (will be empty)
        """
        if name is MISSING and create_subfolder==True:
            name = _projectname()
        os.mkdir(Path(ROOT, name, 'src'))
        with open(Path(ROOT, name, 'main.py'), 'w') as f:
            pass
        if readme:
            with open(Path(ROOT, name, 'README.md'), 'w') as f:
                pass
        if requirements:
            with open(Path(ROOT, name, 'requirements.txt'), 'w') as f:
                f.write("team4958_customs")
        if license_file:
            with open(Path(ROOT, name, 'LICENSE'), 'w') as f:
                pass
    
    def _fromdict(obj:dict, path=MISSING):
        """
        `NEVER FUCKING CALL THIS INSIDE YOUR CODE`
        ------------
        \n
        this will spawn your whole structure directly in current work directory!\n
        `use 'fromdict' method instead`\n
        --------\n
        builds a project structure from dict
        ----
        \n
        -------\n
        example_dict = {
            'folder_name':{
                #feel free to make as much substructures as you need
            },
            'filename.extension':['line1', 'line2', 'line3', etc.],
            'filename2.extension':'any text to put inside the file just as string'
        }
        """
        if path is MISSING:
            path = ROOT
        for (key, val) in obj.items():
            if type(val) is dict:
                os.mkdir(pathlib.Path(path, key))
                newpath = pathlib.Path(path, key)
                Build._fromdict(val, newpath)
            elif type(val) is list or type(val) is str:
                with open(pathlib.Path(path, key), 'w') as f:
                    if type(val) is list:
                        if val!=[]:
                            for line in val:
                                f.writelines(line)
                    else:
                        f.write(val)
            else:
                print(f'"{key}" type not specified as file or folder, skipping...')
    
    def fromdict(preset:dict, name: str=MISSING, create_subfolder=True):
        """
        builds a project structure from dict
        ----
        \n
        ---------\n
        preset: your own structure preset presented as dictionary
        \n
        example_dict = {\n
            'folder_name':{\n
                #feel free to make as much substructures as you need\n
            },\n
            'filename.extension':['line1', 'line2', 'line3', etc.],\n
            'filename2.extension':'any text to put inside the file just as string'\n
        }\n
        -------\n
        name: your project's name (required and has its point only if 'create_subfolder' is True)\n
        create_subfolder: whether you wish to create everything inside a subfolder\n
        """
        if name is MISSING and create_subfolder==True:
            name = _projectname()
            os.mkdir(Path(ROOT, name))
            proj_path = Path(ROOT, name)
        else:
            proj_path = ROOT
        Build._fromdict(preset, path=proj_path)
    
    def deafult(name: str=MISSING, create_subfolder=True):
        """
        builds a project structure from built-in deafult template
        ----
        \n
        ---------\n
        name: your project's name (required and has its point only if 'create_subfolder' is True)\n
        create_subfolder: whether you wish to create everything inside a subfolder\n
        """
        if name is MISSING and create_subfolder==True:
            name = _projectname()
            os.mkdir(Path(ROOT, name))
            proj_path = Path(ROOT, name)
        else:
            proj_path = ROOT
        Build._fromdict(deafult_preset, path=proj_path)
        
    # def disnake_bot(name: str=MISSING, create_subfolder=True):
    #     """
    #     builds a disnake bot project structure from built-in template
    #     ----
    #     \n
    #     ---------\n
    #     name: your project's name (required and has its point only if 'create_subfolder' is True)\n
    #     create_subfolder: whether you wish to create everything inside a subfolder\n
    #     """
    #     if name is MISSING and create_subfolder==True:
    #         name = _projectname()
    #         os.mkdir(Path(ROOT, name))
    #         proj_path = Path(ROOT, name)
    #     else:
    #         proj_path = ROOT
    #     Build._fromdict(disnake_preset, path=proj_path)
